@@ -72,7 +72,7 @@ monomorphise err mono ty = return ty
 -- | Test all properties in the current module, using a custom
 -- 'quickCheck' function. The same caveats as with 'quickCheckAll'
 -- apply.
--- 
+--
 -- @$'forAllProperties'@ has type @('Property' -> 'IO' 'Result') -> 'IO' 'Bool'@.
 -- An example invocation is @$'forAllProperties' 'quickCheckResult'@,
 -- which does the same thing as @$'quickCheckAll'@.
@@ -86,7 +86,7 @@ forAllProperties = do
       quickCheckOne :: (Int, String) -> Q [Exp]
       quickCheckOne (l, x) = do
         exists <- return False `recover` (reify (mkName x) >> return True)
-        if exists then sequence [ [| ($(stringE $ x ++ " on " ++ filename ++ ":" ++ show l),
+        if exists then sequence [ [| ($(stringE $ x ++ " from " ++ filename ++ ":" ++ show l),
                                      property $(mono (mkName x))) |] ]
          else return []
   [| runQuickCheckAll $(fmap (ListE . concat) (mapM quickCheckOne idents)) |]
@@ -98,9 +98,9 @@ forAllProperties = do
 --
 -- Using 'quickCheckAll' interactively doesn't work.
 -- Instead, add a definition to your module along the lines of
--- 
+--
 -- > runTests = $quickCheckAll
--- 
+--
 -- and then execute @runTests@.
 quickCheckAll :: Q Exp
 quickCheckAll = [| $(forAllProperties) quickCheckResult |]
@@ -115,7 +115,9 @@ runQuickCheckAll ps qc =
   fmap and . forM ps $ \(xs, p) -> do
     putStrLn $ "=== " ++ xs ++ " ==="
     r <- qc p
+    putStrLn ""
     return $ case r of
       Success { } -> True
       Failure { } -> False
       NoExpectedFailure { } -> False
+      GaveUp { } -> False
