@@ -1,15 +1,16 @@
 module Test.QuickCheck.Arbitrary
-  ( 
+  (
   -- * Arbitrary and CoArbitrary classes
     Arbitrary(..)
   , CoArbitrary(..)
-  
+
   -- ** Helper functions for implementing arbitrary
   , arbitrarySizedIntegral        -- :: Num a => Gen a
   , arbitraryBoundedIntegral      -- :: (Bounded a, Integral a) => Gen a
   , arbitrarySizedBoundedIntegral -- :: (Bounded a, Integral a) => Gen a
   , arbitrarySizedFractional      -- :: Fractional a => Gen a
   , arbitraryBoundedRandom        -- :: (Bounded a, Random a) => Gen a
+  , arbitraryBoundedEnum          -- :: (Bounded a, Enum a) => Gen a
   -- ** Helper functions for implementing shrink
   , shrinkNothing            -- :: a -> [a]
   , shrinkList               -- :: (a -> [a]) -> [a] -> [[a]]
@@ -20,7 +21,8 @@ module Test.QuickCheck.Arbitrary
   , coarbitraryIntegral      -- :: Integral a => a -> Gen b -> Gen b
   , coarbitraryReal          -- :: Real a => a -> Gen b -> Gen b
   , coarbitraryShow          -- :: Show a => a -> Gen b -> Gen b
-  
+  , coarbitraryEnum          -- :: Enum a => a -> Gen b -> Gen b
+
   -- ** Generators which use arbitrary
   , vector      -- :: Arbitrary a => Int -> Gen [a]
   , orderedList -- :: (Ord a, Arbitrary a) => Gen [a]
@@ -50,6 +52,11 @@ import Data.Char
   , isSpace
   )
 
+import Data.Fixed
+  ( Fixed
+  , HasResolution
+  )
+
 import Data.Ratio
   ( Ratio
   , (%)
@@ -88,7 +95,7 @@ class Arbitrary a where
   -- | A generator for values of the given type.
   arbitrary :: Gen a
   arbitrary = error "no default generator"
-  
+
   -- | Produces a (possibly) empty list of all the possible
   -- immediate shrinks of the given value.
   shrink :: a -> [a]
@@ -107,18 +114,24 @@ instance Arbitrary Bool where
   shrink True = [False]
   shrink False = []
 
+instance Arbitrary Ordering where
+  arbitrary = arbitraryBoundedEnum
+  shrink GT = [EQ, LT]
+  shrink LT = [EQ]
+  shrink EQ = []
+
 instance Arbitrary a => Arbitrary (Maybe a) where
   arbitrary = frequency [(1, return Nothing), (3, liftM Just arbitrary)]
-  
+
   shrink (Just x) = Nothing : [ Just x' | x' <- shrink x ]
   shrink _        = []
 
 instance (Arbitrary a, Arbitrary b) => Arbitrary (Either a b) where
   arbitrary = oneof [liftM Left arbitrary, liftM Right arbitrary]
-  
+
   shrink (Left x)  = [ Left  x' | x' <- shrink x ]
   shrink (Right y) = [ Right y' | y' <- shrink y ]
-  
+
 instance Arbitrary a => Arbitrary [a] where
   arbitrary = sized $ \n ->
     do k <- choose (0,n)
@@ -134,7 +147,7 @@ shrinkList shr xs = concat [ removes k n xs | k <- takeWhile (>0) (iterate (`div
 
   shrinkOne []     = []
   shrinkOne (x:xs) = [ x':xs | x'  <- shr x ]
-                  ++ [ x:xs' | xs' <- shrinkOne xs ] 
+                  ++ [ x:xs' | xs' <- shrinkOne xs ]
 
   removes k n xs
     | k > n     = []
@@ -161,44 +174,48 @@ instance (RealFloat a, Arbitrary a) => Arbitrary (Complex a) where
   shrink (x :+ y) = [ x' :+ y | x' <- shrink x ] ++
                     [ x :+ y' | y' <- shrink y ]
 
+instance HasResolution a => Arbitrary (Fixed a) where
+    arbitrary = arbitrarySizedFractional
+    shrink    = shrinkRealFrac
+
 instance (Arbitrary a, Arbitrary b)
       => Arbitrary (a,b)
  where
   arbitrary = liftM2 (,) arbitrary arbitrary
-  
+
   shrink (x,y) = [ (x',y) | x' <- shrink x ]
               ++ [ (x,y') | y' <- shrink y ]
-              
+
 instance (Arbitrary a, Arbitrary b, Arbitrary c)
       => Arbitrary (a,b,c)
  where
   arbitrary = liftM3 (,,) arbitrary arbitrary arbitrary
-  
+
   shrink (x,y,z) = [ (x',y,z) | x' <- shrink x ]
                 ++ [ (x,y',z) | y' <- shrink y ]
                 ++ [ (x,y,z') | z' <- shrink z ]
-              
+
 instance (Arbitrary a, Arbitrary b, Arbitrary c, Arbitrary d)
       => Arbitrary (a,b,c,d)
  where
   arbitrary = liftM4 (,,,) arbitrary arbitrary arbitrary arbitrary
-  
+
   shrink (w,x,y,z) = [ (w',x,y,z) | w' <- shrink w ]
                   ++ [ (w,x',y,z) | x' <- shrink x ]
                   ++ [ (w,x,y',z) | y' <- shrink y ]
                   ++ [ (w,x,y,z') | z' <- shrink z ]
-              
+
 instance (Arbitrary a, Arbitrary b, Arbitrary c, Arbitrary d, Arbitrary e)
       => Arbitrary (a,b,c,d,e)
  where
   arbitrary = liftM5 (,,,,) arbitrary arbitrary arbitrary arbitrary arbitrary
-  
+
   shrink (v,w,x,y,z) = [ (v',w,x,y,z) | v' <- shrink v ]
                     ++ [ (v,w',x,y,z) | w' <- shrink w ]
                     ++ [ (v,w,x',y,z) | x' <- shrink x ]
                     ++ [ (v,w,x,y',z) | y' <- shrink y ]
                     ++ [ (v,w,x,y,z') | z' <- shrink z ]
-              
+
 -- typical instance for primitive (numerical) types
 
 instance Arbitrary Integer where
@@ -262,7 +279,7 @@ instance Arbitrary Char where
               , not (isSpace a)
               , a)
               )
-    
+
 instance Arbitrary Float where
   arbitrary = arbitrarySizedFractional
   shrink    = shrinkRealFrac
@@ -308,6 +325,14 @@ arbitraryBoundedIntegral =
 arbitraryBoundedRandom :: (Bounded a, Random a) => Gen a
 arbitraryBoundedRandom = choose (minBound,maxBound)
 
+-- | Generates an element of a bounded enumeration.
+arbitraryBoundedEnum :: (Bounded a, Enum a) => Gen a
+arbitraryBoundedEnum =
+  do let mn = minBound
+         mx = maxBound `asTypeOf` mn
+     n <- choose (fromEnum mn, fromEnum mx)
+     return (toEnum n `asTypeOf` mn)
+
 -- | Generates an integral number from a bounded domain. The number is
 -- chosen from the entire range of the type, but small numbers are
 -- generated more often than big numbers. Inspired by demands from
@@ -325,13 +350,13 @@ arbitrarySizedBoundedIntegral =
 
 -- ** Helper functions for implementing shrink
 
--- | Returns no shrinking alternatives. 
+-- | Returns no shrinking alternatives.
 shrinkNothing :: a -> [a]
 shrinkNothing _ = []
 
 -- | Shrink an integral number.
 shrinkIntegral :: Integral a => a -> [a]
-shrinkIntegral x = 
+shrinkIntegral x =
   nub $
   [ -x
   | x < 0, -x > x
@@ -368,9 +393,9 @@ shrinkRealFrac x =
 class CoArbitrary a where
   -- | Used to generate a function of type @a -> c@. The implementation
   -- should use the first argument to perturb the random generator
-  -- given as the second argument. the returned generator 
+  -- given as the second argument. the returned generator
   -- is then used to generate the function result.
-  -- You can often use 'variant' and '><' to implement 
+  -- You can often use 'variant' and '><' to implement
   -- 'coarbitrary'.
   coarbitrary :: a -> Gen c -> Gen c
 
@@ -382,12 +407,12 @@ class CoArbitrary a where
   coarbitrary{| a :+: b |} (Inr y)   = variant (-1) . coarbitrary y
 -}
 
--- | Combine two generator perturbing functions, for example the 
+-- | Combine two generator perturbing functions, for example the
 -- results of calls to 'variant' or 'coarbitrary'.
-(><) :: (Gen a -> Gen a) -> (Gen a -> Gen a) -> (Gen a -> Gen a) 
+(><) :: (Gen a -> Gen a) -> (Gen a -> Gen a) -> (Gen a -> Gen a)
 (><) f g gen =
   do n <- arbitrary
-     (g . variant (n :: Int) . f) gen 
+     (g . variant (n :: Int) . f) gen
 
 -- for the sake of non-GHC compilers, I have added definitions
 -- for coarbitrary here.
@@ -396,7 +421,7 @@ instance (Arbitrary a, CoArbitrary b) => CoArbitrary (a -> b) where
   coarbitrary f gen =
     do xs <- arbitrary
        coarbitrary (map f xs) gen
-  
+
 instance CoArbitrary () where
   coarbitrary _ = id
 
@@ -404,6 +429,11 @@ instance CoArbitrary Bool where
   coarbitrary False = variant 0
   coarbitrary True  = variant (-1)
 
+instance CoArbitrary Ordering where
+  coarbitrary GT = variant 1
+  coarbitrary EQ = variant 0
+  coarbitrary LT = variant (-1)
+
 instance CoArbitrary a => CoArbitrary (Maybe a) where
   coarbitrary Nothing  = variant 0
   coarbitrary (Just x) = variant (-1) . coarbitrary x
@@ -411,7 +441,7 @@ instance CoArbitrary a => CoArbitrary (Maybe a) where
 instance (CoArbitrary a, CoArbitrary b) => CoArbitrary (Either a b) where
   coarbitrary (Left x)  = variant 0    . coarbitrary x
   coarbitrary (Right y) = variant (-1) . coarbitrary y
-  
+
 instance CoArbitrary a => CoArbitrary [a] where
   coarbitrary []     = variant 0
   coarbitrary (x:xs) = variant (-1) . coarbitrary (x,xs)
@@ -419,6 +449,9 @@ instance CoArbitrary a => CoArbitrary [a] where
 instance (Integral a, CoArbitrary a) => CoArbitrary (Ratio a) where
   coarbitrary r = coarbitrary (numerator r,denominator r)
 
+instance HasResolution a => CoArbitrary (Fixed a) where
+  coarbitrary = coarbitraryReal
+
 instance (RealFloat a, CoArbitrary a) => CoArbitrary (Complex a) where
   coarbitrary (x :+ y) = coarbitrary x >< coarbitrary y
 
@@ -427,14 +460,14 @@ instance (CoArbitrary a, CoArbitrary b)
  where
   coarbitrary (x,y) = coarbitrary x
                    >< coarbitrary y
-              
+
 instance (CoArbitrary a, CoArbitrary b, CoArbitrary c)
       => CoArbitrary (a,b,c)
  where
   coarbitrary (x,y,z) = coarbitrary x
                      >< coarbitrary y
                      >< coarbitrary z
-              
+
 instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d)
       => CoArbitrary (a,b,c,d)
  where
@@ -442,7 +475,7 @@ instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d)
                        >< coarbitrary y
                        >< coarbitrary z
                        >< coarbitrary v
-              
+
 instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d, CoArbitrary e)
       => CoArbitrary (a,b,c,d,e)
  where
@@ -451,7 +484,7 @@ instance (CoArbitrary a, CoArbitrary b, CoArbitrary c, CoArbitrary d, CoArbitrar
                          >< coarbitrary z
                          >< coarbitrary v
                          >< coarbitrary w
-              
+
 -- typical instance for primitive (numerical) types
 
 instance CoArbitrary Integer where
@@ -510,6 +543,10 @@ coarbitraryReal x = coarbitrary (toRational x)
 coarbitraryShow :: Show a => a -> Gen b -> Gen b
 coarbitraryShow x = coarbitrary (show x)
 
+-- | A 'coarbitrary' implementation for enums.
+coarbitraryEnum :: Enum a => a -> Gen b -> Gen b
+coarbitraryEnum = variant . fromEnum
+
 --------------------------------------------------------------------------
 -- ** arbitrary generators
 
