@@ -1,12 +1,14 @@
+-- | Test case generation.
 module Test.QuickCheck.Gen where
 
 --------------------------------------------------------------------------
 -- imports
 
 import System.Random
-  ( RandomGen(..)
-  , Random(..)
+  ( Random
   , StdGen
+  , randomR
+  , split
   , newStdGen
   )
 
@@ -19,14 +21,6 @@ import Control.Applicative
   ( Applicative(..)
   )
 
-import Control.Monad.Reader()
-  -- needed for "instance Monad (a ->)"
-  
-  -- 2005-09-16:
-  -- GHC gives a warning for this. I reported this as a bug. /Koen
-
--- * Test case generation
-
 --------------------------------------------------------------------------
 -- ** Generator type
 
@@ -43,7 +37,7 @@ instance Applicative Gen where
 instance Monad Gen where
   return x =
     MkGen (\_ _ -> x)
-  
+
   MkGen m >>= k =
     MkGen (\r n ->
       let (r1,r2)  = split r
@@ -56,7 +50,7 @@ instance Monad Gen where
 
 -- | Modifies a generator using an integer seed.
 variant :: Integral n => n -> Gen a -> Gen a
-variant k (MkGen m) = MkGen (\r n -> m (var k r) n)
+variant k0 (MkGen m) = MkGen (\r n -> m (var k0 r) n)
  where
   var k = (if k == k' then id  else var k')
         . (if even k  then fst else snd)
@@ -77,20 +71,20 @@ resize n (MkGen m) = MkGen (\r _ -> m r n)
 choose :: Random a => (a,a) -> Gen a
 choose rng = MkGen (\r _ -> let (x,_) = randomR rng r in x)
 
--- | Promotes a generator to a generator of monadic values.
+-- | Promotes a monadic generator to a generator of monadic values.
 promote :: Monad m => m (Gen a) -> Gen (m a)
 promote m = MkGen (\r n -> liftM (\(MkGen m') -> m' r n) m)
 
 -- | Generates some example values.
 sample' :: Gen a -> IO [a]
 sample' (MkGen m) =
-  do rnd <- newStdGen
+  do rnd0 <- newStdGen
      let rnds rnd = rnd1 : rnds rnd2 where (rnd1,rnd2) = split rnd
-     return [(m r n) | (r,n) <- rnds rnd `zip` [0,2..20] ]
+     return [(m r n) | (r,n) <- rnds rnd0 `zip` [0,2..20] ]
 
 -- | Generates some example values and prints them to 'stdout'.
 sample :: Show a => Gen a -> IO ()
-sample g = 
+sample g =
   do cases <- sample' g
      sequence_ (map print cases)
 
@@ -107,7 +101,7 @@ gen `suchThat` p =
 
 -- | Tries to generate a value that satisfies a predicate.
 suchThatMaybe :: Gen a -> (a -> Bool) -> Gen (Maybe a)
-gen `suchThatMaybe` p = sized (try 0 . max 1) 
+gen `suchThatMaybe` p = sized (try 0 . max 1)
  where
   try _ 0 = return Nothing
   try k n = do x <- resize (2*k+n) gen
@@ -123,9 +117,9 @@ oneof gs = choose (0,length gs - 1) >>= (gs !!)
 -- The input list must be non-empty.
 frequency :: [(Int, Gen a)] -> Gen a
 frequency [] = error "QuickCheck.frequency used with empty list"
-frequency xs = choose (1, tot) >>= (`pick` xs)
+frequency xs0 = choose (1, tot) >>= (`pick` xs0)
  where
-  tot = sum (map fst xs)
+  tot = sum (map fst xs0)
 
   pick n ((k,x):xs)
     | n <= k    = x
@@ -150,7 +144,7 @@ growingElements xs = sized $ \n -> elements (take (1 `max` size n) xs)
    log'   = round . log . fromIntegral
    size n = (log' n + 1) * k `div` log' mx
 
-{- WAS:                                                                              
+{- WAS:
 growingElements xs = sized $ \n -> elements (take (1 `max` (n * k `div` 100)) xs)
  where
   k = length xs
@@ -163,7 +157,7 @@ listOf gen = sized $ \n ->
   do k <- choose (0,n)
      vectorOf k gen
 
--- | Generates a non-empty list of random length. The maximum length 
+-- | Generates a non-empty list of random length. The maximum length
 -- depends on the size parameter.
 listOf1 :: Gen a -> Gen [a]
 listOf1 gen = sized $ \n ->
