@@ -6,7 +6,7 @@ module Test.QuickCheck.Property where
 
 import Test.QuickCheck.Gen
 import Test.QuickCheck.Arbitrary
-import Test.QuickCheck.Text( showErr, putLine )
+import Test.QuickCheck.Text( showErr, isOneLine, putLine )
 import Test.QuickCheck.Exception
 import Test.QuickCheck.State
 
@@ -60,18 +60,20 @@ type Property = Gen Prop
 -- | The class of things which can be tested, i.e. turned into a property.
 class Testable prop where
   property :: prop -> Property
-
-instance Testable () where
-  property _ = property rejected
+  exhaustive :: prop -> Bool
+  exhaustive _ = False
 
 instance Testable Bool where
   property = property . liftBool
+  exhaustive _ = True
 
 instance Testable Result where
   property = return . MkProp . protectResults . return
+  exhaustive _ = True
 
 instance Testable Prop where
   property (MkProp r) = return . MkProp . ioRose . return $ r
+  exhaustive _ = True
 
 instance Testable prop => Testable (Gen prop) where
   property mp = do p <- mp; property p
@@ -107,7 +109,7 @@ joinRose (MkRose (IORose rm) rs) = IORose $ do r <- rm; return (joinRose (MkRose
 joinRose (MkRose (MkRose x ts) tts) =
   -- first shrinks outer quantification; makes most sense
   MkRose x (map joinRose tts ++ ts)
-  -- first shrinks inner quantification
+  -- first shrinks inner quantification: terrible
   --MkRose x (ts ++ map joinRose tts)
 
 instance Functor Rose where
@@ -159,6 +161,7 @@ data Result
   , expect      :: Bool           -- ^ indicates what the expected result of the property is
   , reason      :: String         -- ^ a message indicating what went wrong
   , interrupted :: Bool           -- ^ indicates if the test case was cancelled by pressing ^C
+  , abort       :: Bool           -- ^ if True, the test should not be repeated
   , stamp       :: [(String,Int)] -- ^ the collected values for this test case
   , callbacks   :: [Callback]     -- ^ the callbacks for this test case
   }
@@ -170,18 +173,26 @@ result =
   , expect      = True
   , reason      = ""
   , interrupted = False
+  , abort       = False
   , stamp       = []
   , callbacks   = []
   }
 
 exception :: String -> AnException -> Result
-exception msg err = failed{ reason = msg ++ ": '" ++ showErr err ++ "'",
-                            interrupted = isInterrupt err }
+exception msg err
+  | isDiscard err = rejected
+  | otherwise = failed{ reason = formatException msg err,
+                        interrupted = isInterrupt err }
+
+formatException :: String -> AnException -> String
+formatException msg err = msg ++ ":" ++ format (show err)
+  where format xs | isOneLine xs = " '" ++ xs ++ "'"
+                  | otherwise = "\n" ++ unlines [ "  " ++ l | l <- lines xs ]
 
 protectResult :: IO Result -> IO Result
 protectResult = protect (exception "Exception")
 
-succeeded :: Result 
+succeeded :: Result
 succeeded = result{ ok = Just True }
 
 failed :: Result
@@ -240,8 +251,13 @@ callback cb = mapTotalResult (\res -> res{ callbacks = cb : callbacks res })
 -- | Prints a message to the terminal as part of the counterexample.
 printTestCase :: Testable prop => String -> prop -> Property
 printTestCase s =
-  callback $ PostFinalFailure Counterexample $ \st _res ->
-    putLine (terminal st) s
+  callback $ PostFinalFailure Counterexample $ \st _res -> do
+    res <- tryEvaluateIO (putLine (terminal st) s)
+    case res of
+      Left err ->
+        putLine (terminal st) (formatException "Exception thrown by generator" err)
+      Right () ->
+        return ()
 
 -- | Performs an 'IO' action after the last failure of a property.
 whenFail :: Testable prop => IO () -> prop -> Property
@@ -250,7 +266,7 @@ whenFail m =
     m
 
 -- | Performs an 'IO' action every time a property fails. Thus,
--- if shrinking is done, this can be used to keep track of the 
+-- if shrinking is done, this can be used to keep track of the
 -- failures along the way.
 whenFail' :: Testable prop => IO () -> prop -> Property
 whenFail' m =
@@ -275,6 +291,10 @@ verbose = mapResult (\res -> res { callbacks = newCallbacks (callbacks res) ++ c
 expectFailure :: Testable prop => prop -> Property
 expectFailure = mapTotalResult (\res -> res{ expect = False })
 
+-- | Modifies a property so that it only will be tested once.
+once :: Testable prop => prop -> Property
+once = mapTotalResult (\res -> res{ abort = True })
+
 -- | Attaches a label to a property. This is used for reporting
 -- test case distribution.
 label :: Testable prop => String -> prop -> Property
@@ -287,7 +307,7 @@ collect :: (Show a, Testable prop) => a -> prop -> Property
 collect x = label (show x)
 
 -- | Conditionally labels test case.
-classify :: Testable prop => 
+classify :: Testable prop =>
             Bool    -- ^ @True@ if the test case should be labelled.
          -> String  -- ^ Label.
          -> prop -> Property
@@ -295,7 +315,7 @@ classify b s = cover b 0 s
 
 -- | Checks that at least the given proportion of the test cases belong
 -- to the given class.
-cover :: Testable prop => 
+cover :: Testable prop =>
          Bool   -- ^ @True@ if the test case belongs to the class.
       -> Int    -- ^ The required percentage (0-100) of test cases.
       -> String -- ^ Label for the test case class.
@@ -306,9 +326,10 @@ cover True n s = n `seq` s `listSeq` (mapTotalResult $ \res -> res { stamp = (s,
 cover False _ _ = property
 
 -- | Implication for properties: The resulting property holds if
--- the first argument is 'False', or if the given property holds.
+-- the first argument is 'False' (in which case the test case is discarded),
+-- or if the given property holds.
 (==>) :: Testable prop => Bool -> prop -> Property
-False ==> _ = property ()
+False ==> _ = property rejected
 True  ==> p = property p
 
 -- | Considers a property failed if it does not complete within
@@ -360,7 +381,7 @@ p1 .&&. p2 = conjoin [property p1, property p2]
 
 -- | Take the conjunction of several properties.
 conjoin :: Testable prop => [prop] -> Property
-conjoin ps = 
+conjoin ps =
   do roses <- mapM (fmap unProp . property) ps
      return (MkProp (conj [] roses))
  where
@@ -389,7 +410,7 @@ p1 .||. p2 = disjoin [property p1, property p2]
 
 -- | Take the disjunction of several properties.
 disjoin :: Testable prop => [prop] -> Property
-disjoin ps = 
+disjoin ps =
   do roses <- mapM (fmap unProp . property) ps
      return (MkProp (foldr disj (MkRose failed []) roses))
  where
